<?php
define('FPDF_FONTPATH','fonts');
require('fpdf.php');

// class to display text with a traditional line printer green bar background. 
// Traditionally this was 11in x 14in paper with sprocket holes holding 66 lines of 132 character lines with a fixed width font
// so we emulate this by scaling to one page per sheet in Landscape mode or two per sheet in Portrait.
//

ini_set('memory_limit', '256M');	// required to handle large input files > 16MB as output is around 4 x larger

class PDF_GreenBar extends FPDF {
	protected $cur_line;		// current line
	protected $Page;		// current Page being printed
	protected $line_height;		// in points
	protected $bar_height;		// normally one sixth of each line height
	protected $y_shimmy;		// amount we shift green bars down so actual lines overlay
	protected $numfontsize;		// for small numbers of each line
	protected $pagespersheet;	// 1,2 or 4
	protected $linesperpage;	// default is 66 lines per page
	protected $sprockets;		// number of sprockets on page
	protected $colour;		// actual colour of "green" bars
	protected $fontName;		// font required for this PDF
	
	protected $FrameX;		// current frame top left corner
	protected $FrameY;		// current frame top left corner

	protected $FrameTopM;		// frame top and bottom margin
	protected $FrameSideM;		// frame side margins
	protected $FrameH;		// frame height
	protected $FrameW;		// frame width
	protected $BarIndent;		// indent for green bar from FrameX 
	protected $BarLen;		// length of green bar
	protected $SockOffset;		// position of socket mid point on page
	
	function __construct($orientation='L', $size='A4', $font='Courier', $lines=66)
	{
	
		parent::__construct($orientation === "Q" ? "L" : $orientation, 'pt', $size);

		$this->fontName = $font;
		$this->AddFont($font,'');
		
		// set up green bar and counters based on orientation and size
		$this->setColour(192, 230, 192);	// light green
		$this->linesperpage = $lines;
		$this->SetMargins(0, 0);					// left, top margins
		$this->SetAutoPageBreak(false,0);				// bottom margin
		$this->Page = 0;
		$this->sprockets = 22;
		
		switch (strtoupper($orientation)) 
		{
		case 'Q':	$this->PagesPerSheet(4);
				break;
		case 'P': 	$this->PagesPerSheet(2);
				break;
		default:
		case 'L':	$this->PagesPerSheet(1);
				break;
		}
	}
	
	// optional call to set colour of bars and line numbers
	function setColour($rgb)
	{
		$this->colour = $rgb;
	}
	
	// setup Frame options based on the sheet layout
	function PagesPerSheet($pages = 1)
	{
		if ($pages < 1) $pages = 1;
		if ($pages > 2) $pages = 4;
		
		$this->pagespersheet = $pages;
		
		switch ($pages)
		{
		case 1:		$this->FrameH = $this->h;
				$this->FrameW = $this->w;
				break;
		case 2:		$this->FrameH = $this->h / 2;
				$this->FrameW = $this->w;
				break;
		case 4:		$this->FrameH = $this->h / 2;
				$this->FrameW = $this->w / 2;
				break;
		}
		$this->FrameSideM = $this->FrameW * 0.03;			// 3% of page width		
		$this->FrameTopM = 10;		// Margin
		
		$this->line_height = ($this->FrameH - 2 * $this->FrameTopM ) / $this->linesperpage;
		$this->y_shimmy = 0;	

		$this->bar_height = $this->line_height / 5;			// gap between the 6 green bars	
	
		if ($this->fontName == 'Courier') {
			$this->y_shimmy = $this->line_height * .24;		// align bars slightly lower than text on page
		} 
		$this->SetFont($this->fontName, 'B', $this->line_height);	// change '' to 'B' for bold font on 
										// above line required to get fontsize for below!

		$this->numfontsize = $this->FontSize - 2;			// for line numbers on even lines
		$this->BarIndent = $this->FrameSideM + $this->FontSize * 3;	// how far from edges bars and printing start
		$this->BarLen = $this->FrameW - 2 * $this->BarIndent;
		$this->SockOffset = $this->FrameSideM;				// socket column placement
		
	}

	function PDFfromFP($fp)
	{
		$this->PDFFromBuffer(stream_get_contents($fp)); 
	}
	
	function PDFfromFileName($name)
	{
		if (is_readable($name)) {			
			$this->SetTitle(basename($name));
			$this->PDFfromBuffer(file_get_contents($name));
		} else
			$this->Error("Cannot open ".$name." for input");
	}
	function PDFfromBuffer($text)
	{
		$this->Page = 0;			// force new sheet to be issued by resetting Page count
		$this->FF();				// always start new file on a new sheet
		
		$len = strlen($text);
		$out = "";
		$col = 1;				// starting column number for tabs
		$start =  0;				// start position in string buffer
		
		// eat any blank lines, form feeds at end of job or we'll get extra blank page(s) at the end of output
		while ($len > 1 && ($text[$len - 1] == "\f" || $text[$len - 1] == "\n" || $text[$len - 1] == "\r")) {
			$len--;
		}
		// delete leading form feeds to prevent blank page(s) at front of job
		while ($len > $start && $text[$start] == "\f") 
			$start++;

		for ($i = $start ; $i < $len; $i++) {		
			$char = $text[$i];

			switch($char) {	
			case "\f": 
				if (strlen($out) > 0)
					$this->WriteLine($out);
				$this->FF();
				$out = "";
				$col = 1;
				break;
			case "\n":
				$this->WriteLine($out);
				$out = "";
				$col = 1;
				break;
			case "\t":			// deal with tab stop expansion (8)
				$out .= " ";
				$col++;
				while ((($col - 1) % 8) && (col < 133)) {
					$out .= " ";
					$col++;
				}
				break;
			case "\r": 			// overstrike line
				$this->WriteLine($out, false);
				$out = "";
				$col = 1;
				break;
			default:
				if ($col < 133)
					$out .= $char;					
				else if ($col == 133)		// exceeds what a real printer can do
					$out .= chr(182);	// paragraph mark shows line overrun
				$col++;
				break;
			}
		}
		if (strlen($out)> 0)
			$this->WriteLine($out);
	}
		
	
	// protected as does not handle any text expansion for tables or any other special characters	
	protected function WriteLine($text, $addnewline=true)
	{
		$this->Text(
			$this->FrameX + $this->BarIndent, 
			$this->line_height * $this->cur_line + $this->FrameY + $this->FrameTopM, 
			$text );
		if ($addnewline) {
			if ($this->cur_line++ == $this->linesperpage)	// next line would overflow so flush 
				$this->FF();		
		}
	}
		
	protected function FF()					// handle form feed
	{
		if (($this->Page % $this->pagespersheet) == 0) {
			$this->AddPage();		// create new real sheet
	

			// draw sheet dividers if required
			$this->SetDrawColor(0,0,0);
			$this->SetLineWidth(0.25);
			switch ($this->pagespersheet)
			{
			case 4:
				$this->Line($this->w / 2, 0,  $this->w / 2, $this->h);	// vertical
			case 2:
				$this->Line(0, $this->h / 2, $this->w, $this->h / 2);	// horizontal
			}
		}
		// now setup next page within current Frame
		
		$this->DrawPage($this->Page++);				

		$this->SetFontSize($this->line_height);
		$this->SetTextColor(0,0,0);
		$this->SetFont($this->fontName, 'B', $this->line_height);
		$this->cur_line = 1;
	}
	
	protected function DrawPage($page) 
	{
		// draw the actual green bar background of 66 lines with each even line from 2 to 64 having 6 thin lines and 
		// line numbers at each end on each page, called one per line printer page
		// pos is position on page 0 = top left, 1 = bottom left, 2 = top right, 3 = bottom right
		
		switch ($page % $this->pagespersheet) 
		{
		case 0:	$this->FrameX = 0; 		$this->FrameY = 0; 		break;
		case 1:	$this->FrameX = 0;	 	$this->FrameY = $this->h / 2; 	break;
		case 2:	$this->FrameX = $this->w / 2; 	$this->FrameY = 0; 		break;
		case 3:	$this->FrameX = $this->w / 2; 	$this->FrameY = $this->h / 2; 	break;
		}
		
		if ($this->colour[0] != 255 && $this->colour[1] != 255 && $this->colour[2] != 255)	/* not white*/
			$this->drawBars($this->FrameX + $this->BarIndent, $this->FrameY + $this->FrameTopM);
		
		$this->drawSprockets($this->FrameX + $this->SockOffset, 
				$this->FrameX + $this->FrameW - $this->SockOffset, 
				$this->FrameY + $this->FrameTopM );
		
	}
	
	protected function drawBars($leftX, $topY)
	{
		$this->SetDrawColor(...$this->colour);
		$this->SetLineWidth(0.1);
		$this->SetFont('Courier', '', $this->numfontsize); 	// for line numbers
		$this->SetFontSize($this->numfontsize);			// for line numbers
		$this->SetTextColor(...$this->colour);
		
		for ($l = 2; $l <= ($this->linesperpage - 2); $l += 2) {
			$line_y_pos = $topY + $l * $this->line_height + $this->y_shimmy;
			
			$this->Text($leftX - 2 * $this->numfontsize, $line_y_pos, sprintf("%2d", $l));  // left line num
			$this->Text($leftX + $this->BarLen + $this->FontSize, $line_y_pos, $l);		// right line num
			
			for ($s=0 ; $s < 6; $s++) {							// 6 lines
				$y = $line_y_pos - $s * $this->bar_height;
				$this->Line( $leftX, $y, $leftX + $this->BarLen, $y);
			}
		}
	}
	
	protected function drawSprockets($leftX, $rightX, $topY)
	{
		$this->SetFillColor(200);				//gray sprocket holes
		$sIncr = ($this->FrameH - 2 * $this->FrameTopM) / ($this->sprockets);
		$ypos = $topY + $sIncr / 2 - $this->FontSize * 0.375;	// top sprocket 1/2 sprocket gap down
		
		for ($l = 0; $l < $this->sprockets; $l++) {
			$this->Circle($leftX,  $ypos, $this->FontSize * 0.75, 'F');
			$this->Circle($rightX, $ypos, $this->FontSize * 0.75, 'F');
			$ypos += $sIncr; 
		}
	}
	
// BELOW THIS ARE STANDARD ADD ONS TO FPDF

	protected function Circle($x, $y, $r, $style='D')
	{
	    $this->Ellipse($x,$y,$r,$r,$style);
	}

	protected function Ellipse($x, $y, $rx, $ry, $style='D')
	{
	    if($style=='F')
		$op='f';
	    elseif($style=='FD' || $style=='DF')
		$op='B';
	    else
		$op='S';
	    $lx=4/3*(M_SQRT2-1)*$rx;
	    $ly=4/3*(M_SQRT2-1)*$ry;
	    $k=$this->k;
	    $h=$this->h;
	    $this->_out(sprintf('%.2F %.2F m %.2F %.2F %.2F %.2F %.2F %.2F c',
		($x+$rx)*$k,($h-$y)*$k,
		($x+$rx)*$k,($h-($y-$ly))*$k,
		($x+$lx)*$k,($h-($y-$ry))*$k,
		$x*$k,($h-($y-$ry))*$k));
	    $this->_out(sprintf('%.2F %.2F %.2F %.2F %.2F %.2F c',
		($x-$lx)*$k,($h-($y-$ry))*$k,
		($x-$rx)*$k,($h-($y-$ly))*$k,
		($x-$rx)*$k,($h-$y)*$k));
	    $this->_out(sprintf('%.2F %.2F %.2F %.2F %.2F %.2F c',
		($x-$rx)*$k,($h-($y+$ly))*$k,
		($x-$lx)*$k,($h-($y+$ry))*$k,
		$x*$k,($h-($y+$ry))*$k));
	    $this->_out(sprintf('%.2F %.2F %.2F %.2F %.2F %.2F c %s',
		($x+$lx)*$k,($h-($y+$ry))*$k,
		($x+$rx)*$k,($h-($y+$ly))*$k,
		($x+$rx)*$k,($h-$y)*$k,
		$op));
	}


}
?>
